@@ -183,7 +183,7 @@ let tape = TextTape::from_slice(b"hello   = world")?;
 let mut out: Vec<u8> = Vec::new();
 let mut writer = TextWriterBuilder::new().from_writer(&mut out);
 writer.write_tape(&tape)?;
-assert_eq!(&out, b"hello=world\n");
+assert_eq!(&out, b"hello=world");
 # Ok::<(), Box<dyn std::error::Error>>(())
 ```
 
@@ -203,7 +203,7 @@ writer.write_unquoted(b"hello")?;
 writer.write_unquoted(b"world")?;
 writer.write_unquoted(b"foo")?;
 writer.write_unquoted(b"bar")?;
-assert_eq!(&out, b"hello=world\nfoo=bar\n");
+assert_eq!(&out, b"hello=world\nfoo=bar");
 # Ok::<(), Box<dyn std::error::Error>>(())
 ```
 */
