@@ -273,11 +273,11 @@ fn parse_quote_scalar(d: &[u8]) -> Result<(Scalar, &[u8]), Error> {
         let start_ptr = haystack.as_ptr();
         let mut ptr = start_ptr;
         let loop_size = std::mem::size_of::<__m128i>();
-        let end_ptr = haystack[haystack.len()..].as_ptr().sub(loop_size);
+        let end_ptr = start_ptr.add(haystack.len() / loop_size * loop_size);
         let quote = _mm_set1_epi8(b'"' as i8);
         let slash = _mm_set1_epi8(b'\\' as i8);
 
-        while ptr <= end_ptr {
+        while ptr < end_ptr {
             let reg = _mm_loadu_si128(ptr as *const __m128i);
             let slash_found = _mm_cmpeq_epi8(slash, reg);
             if _mm_movemask_epi8(slash_found) != 0 {
@@ -310,7 +310,7 @@ fn split_at_scalar_fallback(d: &[u8]) -> (Scalar, &[u8]) {
     let end_ptr = unsafe { start_ptr.add(d.len()) };
 
     let nind = unsafe { forward_search(start_ptr, end_ptr, is_boundary) };
-    let mut ind = nind.unwrap_or_else(|| d.len());
+    let mut ind = nind.unwrap_or(d.len());
 
     // To work with cases where we have "==bar" we ensure that found index is at least one
     ind = std::cmp::max(ind, 1);
