@@ -4,53 +4,19 @@ use crate::{
 };
 use std::{fmt::Arguments, io::Write, ops::Deref};
 
-/// Customizes writer behavior at a field level
-pub trait WriteVisitor {
-    /// Defines how 32 bit data is written
-    fn visit_f32<W>(&self, writer: W, data: f32) -> Result<(), Error>
-    where
-        W: Write;
-
-    /// Defines how 64 bit data is written
-    fn visit_f64<W>(&self, writer: W, data: f64) -> Result<(), Error>
-    where
-        W: Write;
-}
-
-/// The default writer that will write floating point at full representation
-#[derive(Debug)]
-pub struct DefaultWriteVisitor;
-
-impl WriteVisitor for DefaultWriteVisitor {
-    fn visit_f32<W>(&self, mut writer: W, data: f32) -> Result<(), Error>
-    where
-        W: Write,
-    {
-        write!(writer, "{}", data).map_err(|e| e.into())
-    }
-
-    fn visit_f64<W>(&self, mut writer: W, data: f64) -> Result<(), Error>
-    where
-        W: Write,
-    {
-        write!(writer, "{}", data).map_err(|e| e.into())
-    }
-}
-
 /// Write data in PDS format.
 ///
 /// Instantiated via `TextWriterBuilder`
 #[derive(Debug)]
-pub struct TextWriter<W, V> {
+pub struct TextWriter<W> {
     writer: W,
-    visitor: V,
     scratch: Vec<u8>,
     mode: DepthMode,
     depth: Vec<DepthMode>,
     state: WriteState,
     indent_char: u8,
     indent_factor: u8,
-    just_wrote_line_terminator: bool,
+    needs_line_terminator: bool,
 }
 
 /// Construct a customized text writer
@@ -62,7 +28,7 @@ pub struct TextWriter<W, V> {
 /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
 /// writer.write_unquoted(b"hello")?;
 /// writer.write_unquoted(b"world")?;
-/// assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=world\n");
+/// assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=world");
 /// # Ok(())
 /// # }
 /// ```
@@ -103,10 +69,9 @@ const WRITE_STATE_NEXT: [WriteState; 9] = [
     WriteState::HiddenObjectKey,
 ];
 
-impl<W, V> TextWriter<W, V>
+impl<W> TextWriter<W>
 where
     W: Write,
-    V: WriteVisitor,
 {
     /// Get inner writer, keeping ownership
     pub fn inner(&mut self) -> &mut W {
@@ -127,15 +92,10 @@ where
 
     /// Write out the start of an object
     pub fn write_object_start(&mut self) -> Result<(), Error> {
-        if self.mode == DepthMode::Object || self.state == WriteState::ArrayValueFirst {
-            self.write_preamble()?;
-        } else if self.state != WriteState::ArrayValueFirst {
-            self.write_indent()?;
-        }
-
+        self.write_preamble()?;
         self.writer.write_all(b"{")?;
         self.depth.push(self.mode);
-        self.just_wrote_line_terminator = false;
+        self.needs_line_terminator = true;
         self.mode = DepthMode::Object;
         self.state = WriteState::FirstKey;
         Ok(())
@@ -155,6 +115,7 @@ where
         self.depth.push(self.mode);
         self.mode = DepthMode::Array;
         self.state = WriteState::ArrayValueFirst;
+        self.needs_line_terminator = true;
         Ok(())
     }
 
@@ -172,16 +133,14 @@ where
         }
 
         if old_state != WriteState::ArrayValueFirst && old_state != WriteState::FirstKey {
-            self.write_line_terminator()?;
+            self.writer.write_all(b"\n")?;
             self.write_indent()?;
         } else {
             self.writer.write_all(b" ")?;
         }
 
         self.writer.write_all(b"}")?;
-        self.just_wrote_line_terminator = false;
-        self.write_line_terminator()?;
-
+        self.needs_line_terminator = true;
         Ok(())
     }
 
@@ -196,20 +155,15 @@ where
     /// writer.write_bool(true)?;
     /// writer.write_unquoted(b"foo")?;
     /// writer.write_bool(false)?;
-    /// assert_eq!(&out, b"hello=yes\nfoo=no\n");
+    /// assert_eq!(&out, b"hello=yes\nfoo=no");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_bool(&mut self, data: bool) -> Result<(), Error> {
-        self.write_preamble()?;
         match data {
-            true => self.writer.write_all(b"yes")?,
-            false => self.writer.write_all(b"no")?,
-        };
-
-        self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
-        Ok(())
+            true => write!(self, "yes"),
+            false => write!(self, "no"),
+        }
     }
 
     /// Write an non-equal operator
@@ -222,7 +176,7 @@ where
     /// writer.write_unquoted(b"a")?;
     /// writer.write_operator(Operator::LessThan)?;
     /// writer.write_unquoted(b"b")?;
-    /// assert_eq!(&out, b"a < b\n");
+    /// assert_eq!(&out, b"a < b");
     /// # Ok(())
     /// # }
     /// ```
@@ -245,7 +199,7 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"a")?;
     /// writer.write_unquoted(b"b")?;
-    /// assert_eq!(&out, b"a=b\n");
+    /// assert_eq!(&out, b"a=b");
     /// # Ok(())
     /// # }
     /// ```
@@ -253,7 +207,6 @@ where
         self.write_preamble()?;
         self.writer.write_all(data)?;
         self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
         Ok(())
     }
 
@@ -265,9 +218,6 @@ where
     /// out is assumed to already be in the correct encoding (windows-1252 or
     /// UTF-8)
     ///
-    /// Also if one tries to write a quoted field as a key this method
-    /// will redirect to the unquoted variant
-    ///
     /// ```
     /// use jomini::TextWriterBuilder;
     /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
@@ -275,15 +225,11 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_quoted(b"name")?;
     /// writer.write_quoted(br#"captain "joe" rogers"#)?;
-    /// assert_eq!(&out, b"name=\"captain \\\"joe\\\" rogers\"\n");
+    /// assert_eq!(&out, b"\"name\"=\"captain \\\"joe\\\" rogers\"");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_quoted(&mut self, data: &[u8]) -> Result<(), Error> {
-        if self.expecting_key() {
-            return self.write_unquoted(data);
-        }
-
         self.write_preamble()?;
         let esc_buf = self.scratch.split_off(0);
         let esc = escape(data, esc_buf);
@@ -292,7 +238,6 @@ where
         self.writer.write_all(b"\"")?;
         self.scratch = esc.buffer();
         self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
         Ok(())
     }
 
@@ -305,16 +250,12 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"stability")?;
     /// writer.write_i32(-3);
-    /// assert_eq!(&out, b"stability=-3\n");
+    /// assert_eq!(&out, b"stability=-3");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_i32(&mut self, data: i32) -> Result<(), Error> {
-        self.write_preamble()?;
-        write!(self.writer, "{}", data)?;
-        self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
-        Ok(())
+        write!(self, "{}", data)
     }
 
     /// Write an unsigned 32bit integer.
@@ -326,16 +267,12 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"stability")?;
     /// writer.write_u32(3);
-    /// assert_eq!(&out, b"stability=3\n");
+    /// assert_eq!(&out, b"stability=3");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_u32(&mut self, data: u32) -> Result<(), Error> {
-        self.write_preamble()?;
-        write!(self.writer, "{}", data)?;
-        self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
-        Ok(())
+        write!(self, "{}", data)
     }
 
     /// Write an unsigned 64bit integer.
@@ -347,19 +284,15 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"seed")?;
     /// writer.write_u64(1000000000000);
-    /// assert_eq!(&out, b"seed=1000000000000\n");
+    /// assert_eq!(&out, b"seed=1000000000000");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_u64(&mut self, data: u64) -> Result<(), Error> {
-        self.write_preamble()?;
-        write!(self.writer, "{}", data)?;
-        self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
-        Ok(())
+        write!(self, "{}", data)
     }
 
-    /// Write a 32 bit floating point according to the visitor
+    /// Write a 32 bit floating point at full precision
     ///
     /// ```
     /// use jomini::TextWriterBuilder;
@@ -368,19 +301,32 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"morale")?;
     /// writer.write_f32(4.566);
-    /// assert_eq!(&out, b"morale=4.566\n");
+    /// assert_eq!(&out, b"morale=4.566");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_f32(&mut self, data: f32) -> Result<(), Error> {
-        self.write_preamble()?;
-        self.visitor.visit_f32(&mut self.writer, data)?;
-        self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
-        Ok(())
+        write!(self, "{}", data)
     }
 
-    /// Write a 64 bit floating point according to the visitor
+    /// Write a 32 bit floating point at a given precision
+    ///
+    /// ```
+    /// use jomini::TextWriterBuilder;
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let mut out: Vec<u8> = Vec::new();
+    /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
+    /// writer.write_unquoted(b"morale")?;
+    /// writer.write_f32_precision(4.0, 3);
+    /// assert_eq!(&out, b"morale=4.000");
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn write_f32_precision(&mut self, data: f32, precision: usize) -> Result<(), Error> {
+        write!(self, "{0:.1$}", data, precision)
+    }
+
+    /// Write a 64 bit floating point at full precision
     ///
     /// ```
     /// use jomini::TextWriterBuilder;
@@ -389,16 +335,29 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"strength")?;
     /// writer.write_f64(6790.35609);
-    /// assert_eq!(&out, b"strength=6790.35609\n");
+    /// assert_eq!(&out, b"strength=6790.35609");
     /// # Ok(())
     /// # }
     /// ```
     pub fn write_f64(&mut self, data: f64) -> Result<(), Error> {
-        self.write_preamble()?;
-        self.visitor.visit_f64(&mut self.writer, data)?;
-        self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
-        Ok(())
+        write!(self, "{}", data)
+    }
+
+    /// Write a 64 bit floating point at a given precision
+    ///
+    /// ```
+    /// use jomini::TextWriterBuilder;
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let mut out: Vec<u8> = Vec::new();
+    /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
+    /// writer.write_unquoted(b"morale")?;
+    /// writer.write_f64_precision(4.0, 3);
+    /// assert_eq!(&out, b"morale=4.000");
+    /// # Ok(())
+    /// # }
+    /// ```
+    pub fn write_f64_precision(&mut self, data: f64, precision: usize) -> Result<(), Error> {
+        write!(self, "{0:.1$}", data, precision)
     }
 
     /// Write a header.
@@ -419,7 +378,7 @@ where
     /// writer.write_i32(200)?;
     /// writer.write_i32(50)?;
     /// writer.write_end()?;
-    /// assert_eq!(&out, b"color=rgb {\n  100 200 50\n}\n");
+    /// assert_eq!(&out, b"color=rgb {\n  100 200 50\n}");
     /// # Ok(())
     /// # }
     /// ```
@@ -441,7 +400,7 @@ where
     /// let date = Date::from_ymd(1444, 11, 11);
     /// writer.write_unquoted(b"start")?;
     /// writer.write_date(date.game_fmt())?;
-    /// assert_eq!(&out, b"start=1444.11.11\n");
+    /// assert_eq!(&out, b"start=1444.11.11");
     /// # Ok(())
     /// # }
     /// ```
@@ -460,7 +419,7 @@ where
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_unquoted(b"start")?;
     /// write!(writer, "unknown_{}", 5)?;
-    /// assert_eq!(&out, b"start=unknown_5\n");
+    /// assert_eq!(&out, b"start=unknown_5");
     /// # Ok(())
     /// # }
     /// ```
@@ -468,23 +427,24 @@ where
         self.write_preamble()?;
         self.writer.write_fmt(fmt)?;
         self.write_epilogue()?;
-        self.state = WRITE_STATE_NEXT[self.state as usize];
         Ok(())
     }
 
     fn write_line_terminator(&mut self) -> Result<(), Error> {
-        if !self.just_wrote_line_terminator {
+        if self.needs_line_terminator {
             self.writer.write_all(b"\n")?;
-            self.just_wrote_line_terminator = true;
+            self.needs_line_terminator = false;
         }
 
         Ok(())
     }
 
     fn write_preamble(&mut self) -> Result<(), Error> {
+        let just_wrote_line_terminator = self.needs_line_terminator;
+        self.write_line_terminator()?;
         match self.state {
             WriteState::ArrayValue => {
-                if self.just_wrote_line_terminator {
+                if just_wrote_line_terminator {
                     self.write_indent()?;
                 } else {
                     self.writer.write_all(b" ")?;
@@ -500,22 +460,18 @@ where
                 self.writer.write_all(b"=")?;
             }
             WriteState::ArrayValueFirst | WriteState::FirstKey => {
-                self.write_line_terminator()?;
                 self.write_indent()?;
             }
             _ => {}
         };
 
-        self.just_wrote_line_terminator = false;
         Ok(())
     }
 
-    /// If the next state will be a key, a key should be preceeded by a line terminator
+    /// advance the state
     fn write_epilogue(&mut self) -> Result<(), Error> {
-        if WRITE_STATE_NEXT[self.state as usize] == WriteState::Key {
-            self.write_line_terminator()?;
-        }
-
+        self.state = WRITE_STATE_NEXT[self.state as usize];
+        self.needs_line_terminator = self.state == WriteState::Key;
         Ok(())
     }
 
@@ -538,7 +494,7 @@ where
     /// let mut out: Vec<u8> = Vec::new();
     /// let mut writer = TextWriterBuilder::new().from_writer(&mut out);
     /// writer.write_tape(&tape)?;
-    /// assert_eq!(&out, b"hello=world\n");
+    /// assert_eq!(&out, b"hello=world");
     /// # Ok(())
     /// # }
     /// ```
@@ -561,39 +517,34 @@ where
                     self.write_preamble()?;
                     write!(self.writer, "[[")?;
                     self.writer.write_all(x.as_bytes())?;
-                    self.writer.write_all(b"]")?;
+                    self.writer.write_all(b"]\n")?;
 
                     if let Ok(obj) = value.read_object() {
                         self.write_object_core(obj)?;
+                        self.writer.write_all(b"\n")?;
                         self.write_indent()?;
                     } else {
                         self.write_value(value)?;
                     }
                     self.writer.write_all(b"]")?;
-                    self.just_wrote_line_terminator = false;
-                    self.write_line_terminator()?;
                 }
                 TextToken::UndefinedParameter(x) => {
                     self.write_preamble()?;
                     write!(self.writer, "[[!")?;
                     self.writer.write_all(x.as_bytes())?;
-                    self.writer.write_all(b"]")?;
+                    self.writer.write_all(b"]\n")?;
 
                     if let Ok(obj) = value.read_object() {
                         self.write_object_core(obj)?;
+                        self.writer.write_all(b"\n")?;
                         self.write_indent()?;
                     } else {
                         self.write_value(value)?;
                     }
                     self.writer.write_all(b"]")?;
-                    self.just_wrote_line_terminator = false;
-                    self.write_line_terminator()?;
                 }
                 TextToken::Quoted(x) => {
-                    // quoted keys really shouldn't happen but when they do
-                    // we should make sure to preserve them. This is different
-                    // behavior than writing by hand.
-                    self.force_write_quotes(x.as_bytes())?;
+                    self.write_escaped_quotes(x.as_bytes())?;
                     if let Some(op) = op {
                         self.write_operator(op)?;
                     }
@@ -614,14 +565,13 @@ where
         Ok(())
     }
 
-    fn force_write_quotes(&mut self, x: &[u8]) -> Result<(), Error> {
-        let mut scratch = self.scratch.split_off(0);
-        scratch.clear();
-        scratch.push(b'"');
-        scratch.extend_from_slice(x);
-        scratch.push(b'"');
-        self.write_unquoted(&scratch)?;
-        self.scratch = scratch;
+    // For when the data is already escaped
+    fn write_escaped_quotes(&mut self, x: &[u8]) -> Result<(), Error> {
+        self.write_preamble()?;
+        self.writer.write_all(b"\"")?;
+        self.writer.write_all(x)?;
+        self.writer.write_all(b"\"")?;
+        self.write_epilogue()?;
         Ok(())
     }
 
@@ -641,7 +591,7 @@ where
                 self.write_unquoted(x.as_bytes())?;
             }
             TextToken::Quoted(x) => {
-                self.force_write_quotes(x.as_bytes())?;
+                self.write_escaped_quotes(x.as_bytes())?;
             }
             TextToken::Parameter(_) => unreachable!(),
             TextToken::UndefinedParameter(_) => unreachable!(),
@@ -706,29 +656,19 @@ impl TextWriterBuilder {
     }
 
     /// Construct a text writer from a builder and a writer.
-    pub fn from_writer<R>(&self, writer: R) -> TextWriter<R, DefaultWriteVisitor>
+    pub fn from_writer<R>(&self, writer: R) -> TextWriter<R>
     where
         R: Write,
-    {
-        self.from_writer_visitor(writer, DefaultWriteVisitor)
-    }
-
-    /// Construct a text writer from a builder, writer, and visitor.
-    pub fn from_writer_visitor<R, V>(&self, writer: R, visitor: V) -> TextWriter<R, V>
-    where
-        R: Write,
-        V: WriteVisitor,
     {
         TextWriter {
             writer,
-            visitor,
             scratch: Vec::new(),
             state: WriteState::Key,
             mode: DepthMode::Object,
             depth: Vec::with_capacity(16),
             indent_char: self.indent_char,
             indent_factor: self.indent_factor,
-            just_wrote_line_terminator: false,
+            needs_line_terminator: false,
         }
     }
 }
@@ -809,7 +749,7 @@ fn escape(data: &[u8], mut buffer: Vec<u8>) -> ReuseVec {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::common::{DateHour, PdsDate};
+    use crate::common::{Date, DateHour, PdsDate};
     use std::error::Error;
 
     #[test]
@@ -821,7 +761,7 @@ mod tests {
         assert!(writer.expecting_key());
         writer.write_unquoted(b"foo")?;
         writer.write_unquoted(b"bar")?;
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=world\nfoo=bar\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=world\nfoo=bar");
         Ok(())
     }
 
@@ -836,7 +776,7 @@ mod tests {
         writer.write_end()?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "hello={\n  world foo\n}\n"
+            "hello={\n  world foo\n}"
         );
         Ok(())
     }
@@ -851,7 +791,7 @@ mod tests {
         writer.write_end()?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "hello={\n  \"The Punic Wars\"\n}\n"
+            "hello={\n  \"The Punic Wars\"\n}"
         );
         Ok(())
     }
@@ -864,7 +804,7 @@ mod tests {
         writer.write_bool(true)?;
         writer.write_unquoted(b"foo")?;
         writer.write_bool(false)?;
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=yes\nfoo=no\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=yes\nfoo=no");
         Ok(())
     }
 
@@ -886,7 +826,7 @@ mod tests {
         writer.write_unquoted(b"h")?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "a < b\nc <= f\nd > g\ne >= h\n"
+            "a < b\nc <= f\nd > g\ne >= h"
         );
         Ok(())
     }
@@ -899,39 +839,50 @@ mod tests {
         writer.write_quoted(br#"captain "joe" rogers"#)?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "name=\"captain \\\"joe\\\" rogers\"\n"
+            "\"name\"=\"captain \\\"joe\\\" rogers\""
         );
         Ok(())
     }
 
     #[test]
     fn write_alternative_float_format() -> Result<(), Box<dyn Error>> {
-        struct Eu4Writer;
-        impl WriteVisitor for Eu4Writer {
-            fn visit_f32<W>(&self, mut writer: W, data: f32) -> Result<(), crate::Error>
-            where
-                W: Write,
-            {
-                write!(writer, "{:.3}", data).map_err(|e| e.into())
-            }
-
-            fn visit_f64<W>(&self, mut writer: W, data: f64) -> Result<(), crate::Error>
-            where
-                W: Write,
-            {
-                write!(writer, "{:.5}", data).map_err(|e| e.into())
-            }
-        }
-
         let mut out: Vec<u8> = Vec::new();
-        let mut writer = TextWriterBuilder::new().from_writer_visitor(&mut out, Eu4Writer);
+        let mut writer = TextWriterBuilder::new().from_writer(&mut out);
         writer.write_unquoted(b"morale")?;
-        writer.write_f32(1.0)?;
+        writer.write_f32_precision(1.0, 3)?;
         writer.write_unquoted(b"strength")?;
-        writer.write_f64(1.0)?;
+        writer.write_f64_precision(1.0, 5)?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "morale=1.000\nstrength=1.00000\n"
+            "morale=1.000\nstrength=1.00000"
+        );
+        Ok(())
+    }
+
+    #[test]
+    fn write_primitives() -> Result<(), Box<dyn Error>> {
+        let mut out: Vec<u8> = Vec::new();
+        let mut writer = TextWriterBuilder::new().from_writer(&mut out);
+        writer.write_unquoted(b"primitives")?;
+        writer.write_object_start()?;
+        writer.write_unquoted(b"bool")?;
+        writer.write_bool(true)?;
+        writer.write_unquoted(b"date")?;
+        writer.write_date(Date::from_ymd(1444, 11, 11).game_fmt())?;
+        writer.write_unquoted(b"f32")?;
+        writer.write_f32(1.)?;
+        writer.write_unquoted(b"f64")?;
+        writer.write_f64(2.)?;
+        writer.write_unquoted(b"i32")?;
+        writer.write_i32(3)?;
+        writer.write_unquoted(b"u32")?;
+        writer.write_u32(4)?;
+        writer.write_unquoted(b"u64")?;
+        writer.write_u32(5)?;
+        writer.write_end()?;
+        assert_eq!(
+            std::str::from_utf8(&out).unwrap(),
+            "primitives={\n  bool=yes\n  date=1444.11.11\n  f32=1\n  f64=2\n  i32=3\n  u32=4\n  u64=5\n}"
         );
         Ok(())
     }
@@ -954,7 +905,7 @@ mod tests {
         writer.write_unquoted(b"b")?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "data={\n  id={\n    id=10\n  }\n  name=world\n}\na=b\n"
+            "data={\n  id={\n    id=10\n  }\n  name=world\n}\na=b"
         );
         Ok(())
     }
@@ -977,7 +928,7 @@ mod tests {
         writer.write_unquoted(b"b")?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "data={\n  settings={\n    0 1\n  }\n  name=world\n}\na=b\n"
+            "data={\n  settings={\n    0 1\n  }\n  name=world\n}\na=b"
         );
         Ok(())
     }
@@ -995,7 +946,7 @@ mod tests {
         writer.write_end()?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "data={\n  { }\n  { }\n}\n"
+            "data={\n  { }\n  { }\n}"
         );
         Ok(())
     }
@@ -1018,7 +969,7 @@ mod tests {
 
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "data={\n  {\n    a=b\n  }\n  {\n    c=d\n  }\n}\n"
+            "data={\n  {\n    a=b\n  }\n  {\n    c=d\n  }\n}"
         );
         Ok(())
     }
@@ -1040,7 +991,7 @@ mod tests {
         writer.write_unquoted(b"b")?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "data={\n  10 d=e f=g\n}\na=b\n"
+            "data={\n  10 d=e f=g\n}\na=b"
         );
         Ok(())
     }
@@ -1053,7 +1004,7 @@ mod tests {
         writer.write_array_start()?;
         writer.write_end()?;
 
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "data={ }\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "data={ }");
         Ok(())
     }
 
@@ -1066,7 +1017,7 @@ mod tests {
         assert!(writer.expecting_key());
         writer.write_end()?;
 
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "data={ }\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "data={ }");
         Ok(())
     }
 
@@ -1091,7 +1042,7 @@ mod tests {
         writer.write_unquoted(b"b")?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "data={\n\tsettings={\n\t\t0 1\n\t}\n\tname=world\n}\na=b\n"
+            "data={\n\tsettings={\n\t\t0 1\n\t}\n\tname=world\n}\na=b"
         );
         Ok(())
     }
@@ -1102,7 +1053,7 @@ mod tests {
         let mut writer = TextWriterBuilder::new().from_writer(&mut out);
         writer.write_unquoted(b"hello")?;
         write!(writer, "__unknown_{}", 5)?;
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=__unknown_5\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "hello=__unknown_5");
         Ok(())
     }
 
@@ -1115,7 +1066,7 @@ mod tests {
         writer.write_tape(&tape)?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "army={\n  name=abc\n}\narmy={\n  name=def\n}\n"
+            "army={\n  name=abc\n}\narmy={\n  name=def\n}"
         );
         Ok(())
     }
@@ -1127,7 +1078,7 @@ mod tests {
         let mut out: Vec<u8> = Vec::new();
         let mut writer = TextWriterBuilder::new().from_writer(&mut out);
         writer.write_tape(&tape)?;
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "vals={\n  1 2\n}\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "vals={\n  1 2\n}");
         Ok(())
     }
 
@@ -1138,10 +1089,7 @@ mod tests {
         let mut out: Vec<u8> = Vec::new();
         let mut writer = TextWriterBuilder::new().from_writer(&mut out);
         writer.write_tape(&tape)?;
-        assert_eq!(
-            std::str::from_utf8(&out).unwrap(),
-            "vals={\n  1 a=b d=f\n}\n"
-        );
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "vals={\n  1 a=b d=f\n}");
         Ok(())
     }
 
@@ -1154,7 +1102,7 @@ mod tests {
         writer.write_tape(&tape)?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "color=rgb {\n  100 50 200\n}\n"
+            "color=rgb {\n  100 50 200\n}"
         );
         Ok(())
     }
@@ -1168,7 +1116,7 @@ mod tests {
         writer.write_tape(&tape)?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "generate_advisor={\n  [[scaled_skill]\n  a=b\n  ]\n}\n"
+            "generate_advisor={\n  [[scaled_skill]\n  a=b\n  ]\n}"
         );
         Ok(())
     }
@@ -1182,7 +1130,7 @@ mod tests {
         writer.write_tape(&tape)?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "generate_advisor={\n  [[!scaled_skill]\n  a=b\n  ]\n}\n"
+            "generate_advisor={\n  [[!scaled_skill]\n  a=b\n  ]\n}"
         );
         Ok(())
     }
@@ -1194,7 +1142,7 @@ mod tests {
         let mut out: Vec<u8> = Vec::new();
         let mut writer = TextWriterBuilder::new().from_writer(&mut out);
         writer.write_tape(&tape)?;
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "a=\"\"\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "a=\"\"");
         Ok(())
     }
 
@@ -1205,7 +1153,7 @@ mod tests {
         let mut out: Vec<u8> = Vec::new();
         let mut writer = TextWriterBuilder::new().from_writer(&mut out);
         writer.write_tape(&tape)?;
-        assert_eq!(std::str::from_utf8(&out).unwrap(), "\"\"=a\n");
+        assert_eq!(std::str::from_utf8(&out).unwrap(), "\"\"=a");
         Ok(())
     }
 
@@ -1246,8 +1194,30 @@ mod tests {
         writer.write_date(date2.game_fmt())?;
         assert_eq!(
             std::str::from_utf8(&out).unwrap(),
-            "date=1936.1.1.1\ndate2=1936.1.1.24\n"
+            "date=1936.1.1.1\ndate2=1936.1.1.24"
         );
         Ok(())
     }
+
+    #[test]
+    fn test_non_ascii_tape_write() -> Result<(), Box<dyn Error>> {
+        let data = b"name=\"'Abb\xe2 Osmanoglu\"";
+        let tape = TextTape::from_slice(data)?;
+        let mut out: Vec<u8> = Vec::new();
+        let mut writer = TextWriterBuilder::new().from_writer(&mut out);
+        writer.write_tape(&tape)?;
+        assert_eq!(&out, data);
+        Ok(())
+    }
+
+    #[test]
+    fn test_non_ascii_obj_tape_write() -> Result<(), Box<dyn Error>> {
+        let data = b"obj={\n  name=\"'Abb\xe2 Osmanoglu\"\n}";
+        let tape = TextTape::from_slice(data)?;
+        let mut out: Vec<u8> = Vec::new();
+        let mut writer = TextWriterBuilder::new().from_writer(&mut out);
+        writer.write_tape(&tape)?;
+        assert_eq!(&out, data);
+        Ok(())
+    }
 }
